#include "GameState.h"
#include "SelectState.h"
#include "ThemeManager.h"

GameState::GameState(IStateManager &state_manager, const sf::VideoMode &video_mode, const std::string &window_title) :
        IState(state_manager), IWindowKeeper(config::GAME_VIDEO_MODE, window_title),
        m_menu(), m_score_count_one(std::make_unique<ScoreCount>(1)), m_score_count_two(std::make_unique<ScoreCount>(2)){ }

void GameState::event_handling() {
    sf::Event event{};
    bool mode_made = false;
    while (m_window.pollEvent(event)) {
        if (event.type == sf::Event::Closed)
            m_state_manager.set_next_state(std::make_unique<SelectState>(m_state_manager,  config::GAME_VIDEO_MODE, config::SELECT_LEVEL_TITLE));

        if (event.type == sf::Event::KeyPressed && event.key.code == sf::Keyboard::Z)
            m_context_manager.restore_previous_context();

        if (event.type == sf::Event::KeyPressed && m_context_manager.get_current_context().state  == GameContext::INGAME) {
            if (!mode_made)
                mode_made = process_key_pressed(event.key.code);
        }
    }
}

bool GameState::process_key_pressed(sf::Keyboard::Key code) {
    GameContext& context = m_context_manager.get_current_context();

    if (code == sf::Keyboard::W || code == sf::Keyboard::A || code == sf::Keyboard::S || code == sf::Keyboard::D) {
        m_context_manager.save_current_context();
        switch (code) {
            case sf::Keyboard::W: context.pacman.move(Room::UP); break;
            case sf::Keyboard::S: context.pacman.move(Room::DOWN); break;
            case sf::Keyboard::A: context.pacman.move(Room::LEFT); break;
            case sf::Keyboard::D: context.pacman.move(Room::RIGHT); break;
        }
        return true;
    }

    if (code == sf::Keyboard::Up || code == sf::Keyboard::Down || code == sf::Keyboard::Left || code == sf::Keyboard::Right) {
        if (context.pacman2.get_location() != nullptr) {
            m_context_manager.save_current_context();
            switch (code) {
                case sf::Keyboard::Up: context.pacman2.move(Room::UP); break;
                case sf::Keyboard::Down: context.pacman2.move(Room::DOWN); break;
                case sf::Keyboard::Left: context.pacman2.move(Room::LEFT); break;
                case sf::Keyboard::Right: context.pacman2.move(Room::RIGHT); break;
            }
            return true;
        }
    }
    return false;
}

void GameState::update() {
    std::vector<std::unique_ptr<IGameEvent>> game_events;
    if (m_context_manager.get_current_context().state != GameContext::INGAME) return;
    GameContext& context = m_context_manager.get_current_context();
    for (auto& ptr_entity : context.dynamic_objects) {
        ptr_entity->action();
        IVisitor* ptr_visitor = &(context.pacman);
        auto ptr_event = ptr_entity->accept(ptr_visitor);
        if (ptr_event)  game_events.emplace_back(std::move(ptr_event));

        if (context.pacman2.get_location() != nullptr) {
            IVisitor* ptr_visitor2 = &(context.pacman2);
            auto ptr_event2 = ptr_entity->accept(ptr_visitor2);
            if (ptr_event2)  game_events.emplace_back(std::move(ptr_event2));
        }
    }
    for (auto& static_ptr : context.static_objects) {
        IVisitor* ptr_visitor = &(context.pacman);
        if (auto ptr_event = static_ptr->accept(ptr_visitor)) {
            ptr_event->add_observer(m_score_count_one.get());
            game_events.emplace_back(std::move(ptr_event));
        }

        if (context.pacman2.get_location() != nullptr) {
            IVisitor* ptr_visitor2 = &(context.pacman2);
            if (auto ptr_event2 = static_ptr->accept(ptr_visitor2)) {
                ptr_event2->add_observer(m_score_count_two.get());
                game_events.emplace_back(std::move(ptr_event2));
            }
        }
    }

    if (context.static_objects.empty()) {
        game_events.emplace_back(std::make_unique<WinGame>());
        if (m_score_count_one->get_player_one() > m_score_count_two->get_player_two()) std::cout << "выиграл первый";
        else std::cout << "выиграл второй";
    }

    for (int i = 0; i < game_events.size(); i++)
        game_events[i]->handle(&m_context_manager.get_current_context());

    game_events.clear();
}

void GameState::render() {
    GameContext& context = m_context_manager.get_current_context();
    sf::Color backgroundColor;
    switch (context.state) {
        case GameContext::INGAME:
            backgroundColor = ThemeManager::Instance().getGameBackgroundInGameColor();
            break;
        case GameContext::LOST:
            backgroundColor = ThemeManager::Instance().getGameBackgroundLostColor();
            break;
        case GameContext::WIN:
            backgroundColor = ThemeManager::Instance().getGameBackgroundWinColor();
            break;
        default:
            break;
    }

    m_window.clear(backgroundColor);
    m_menu.draw_into(m_window, *m_score_count_one, *m_score_count_two);
    m_maze.draw_into(m_window);
    context.pacman.draw_into(m_window);
    if (context.pacman2.get_location() != nullptr)
        context.pacman2.draw_into(m_window);
    for (auto& obj :  context.static_objects)
        obj->draw_into(m_window);

    for (auto& el : context.dynamic_objects)
        el->draw_into(m_window);
    m_window.display();
}

bool GameState::do_step() {
    event_handling();
    if (m_context_manager.get_current_context().state == GameContext::INGAME)
        update();
    render();
    return true;
}

void GameState::set_context(GameContext&& context) {
    m_context_manager.reset(std::move(context));
}